<!DOCTYPE html>
<html lang="en" ng-app="scheduler">
<head>
    <meta charset="utf-8">
    <meta http-equiv="X-UA-Compatible" content="IE=edge">
    <meta name="viewport" content="width=device-width, initial-scale=1">

    <meta charset="utf-8" />
    <link href="https://fonts.googleapis.com/css2?family=IBM+Plex+Sans&display=swap" rel="stylesheet">
    <link href="https://fonts.googleapis.com/css2?family=IBM+Plex+Mono&display=swap" rel="stylesheet">

    <base href="/">
</head>
<body>

<nav class="navbar navbar-fixed-top" ng-controller="navbarController">
    <div class="container">
        <div class="pull-left col-sm-4">
            <span>Scheduler status: </span><span class="{{ scheduler.state }}">{{ scheduler.state }}</span>
        </div>
        <div class="col-sm-4 scheduler-controls pull-right" ng-if="capabilities.can_control_scheduler">
            <button class="btn control-btn" ng-if="scheduler.state === 'paused'" ng-click="resume_scheduler()">
                <i class="far fa-play-circle"></i>
                Resume Scheduler
            </button>

<!--            <button class="btn control-btn" ng-if="scheduler.state === 'stopped'" ng-click="start_scheduler()">-->
<!--                <i class="far fa-play-circle"></i>-->
<!--                Start Scheduler-->
<!--            </button>-->

            <button class="btn control-btn control-neutral" ng-if="capabilities.pause_scheduler && scheduler.state === 'running'"
                    ng-click="pause_scheduler()">
                <i class="far fa-pause-circle"></i>
                Pause Scheduler
            </button>

<!--            <button class="btn control-btn control-negative"-->
<!--                    ng-if="capabilities.stop_scheduler && scheduler.state !== 'stopped'"-->
<!--                    ng-click="stop_scheduler()">-->
<!--                <i class="far fa-stop-circle"></i>-->
<!--                Stop Scheduler-->
<!--            </button>-->
        </div>
        <div class="navbar-header">
            <button type="button" class="navbar-toggle collapsed" data-toggle="collapse" data-target="#navbar"
                    aria-expanded="false" aria-controls="navbar">
                <span class="sr-only">Toggle navigation</span>
                <span class="icon-bar"></span>
                <span class="icon-bar"></span>
                <span class="icon-bar"></span>
            </button>
        </div>

        <div id="navbar" class="collapse navbar-collapse">
            <ul class="nav navbar-nav">
                <li ng-class="{ 'selected-section': is_active('/') }"><a href="/">OVERVIEW</a></li>
            </ul>
        </div>
    </div>
</nav>

<div class="main-view">
    <div ng-view></div>
</div>

<footer>
    <div class="container">
        <div class="col-md-6"><span>All times are {{scheduler.timezone}}</span></div>
        <div class="col-md-6 text-right"><span>Type: {{scheduler.class}}</span></div>
    </div>
</footer>

</body>
</html>

