@import "variables";

#plots-container {
    width: 100%;
    position: relative;
    margin-bottom: 70px;

    .upper-axis-container {
        position: fixed;
        top: 160px;
        left: 0;
        overflow: hidden;
        height: 40px;
        width: 100%;
        background-color: @body-bg;
        z-index: 1000;

        #plotly-upper-axis {
            width: 100%;
            height: 90px;
            margin: 0 auto;
            margin-bottom: -1px;
        }
    }

    #plotly-main-plot {
        width: 100%;
        height: 0;
        margin: 0 auto;
        margin-top: 40px;
    }

    .lower-axis-container {
        overflow: hidden;
        height: 44px;
        width: 100%;

        #plotly-lower-axis {
            width: 100%;
            height: 90px;
            margin: 0 auto;
            margin-top: -1px;
        }
    }
}