@body-bg: #1D1E2E;
@body-font: IBM Plex Sans, sans-serif;
@monospaced-font: IBM Plex Mono, monospace;
@navbar-bg: #25263B;


@nav-item-font-color: #96ADFF;
@panel-content-font-color: #E7EBFF;
@panel-header-font-color: #969FCC;
@panel-row-border-color: #474966;

@font-size: 14px;

@navbar-height: 60px;

@scheduler-running: #85FCD5;
@scheduler-not-running: #FFA4A4;

@job-status-executed: #3EBF91;
@job-status-error: #FF6464;
@job-status-submitted: #597EFF;
@job-status-border: #2B3046;

@button-bg: @job-status-submitted;