<div class="modal-header">
    <h3 class="modal-title">Confirm Action</h3>
</div>
<div class="modal-body">
    Are you sure you want to {{ action }}?
</div>
<div class="modal-footer">
    <button class="btn control-btn control-neutral" type="button" ng-click="ok()">OK</button>
    <button class="btn control-btn" type="button" ng-click="cancel()">Cancel</button>
</div>

