@import "variables";

.modal-content {
  background-color: @body-bg;
  font-family: @body-font;

  .modal-header, .modal-footer {
    border-color: @panel-row-border-color;
  }

  & .modal-body {
    font-size: 18px;
  }
}